#include<stdio.h>
#include<string.h>

typedef unsigned long dword;
typedef unsigned char byte;
typedef unsigned int  uint;

#define MOD_ADLER 65521
 

dword Adler32(byte *data,uint size)
{
	uint step;
    dword a=1,b=0;
	while (size) 
    {
        step=size;
		if(step>5550) step=5550;
        size-=step;
        while(step--) a+=*data++,b+=a;
        a=(a&0xffff)+(a>>16)*(65536-MOD_ADLER);
        b=(b&0xffff)+(b>>16)*(65536-MOD_ADLER);
    }
    /* It can be shown that a <= 0x1013a here, so a single subtract will do. */
    if (a>=MOD_ADLER) a-=MOD_ADLER;
    /* It can be shown that b can reach 0xffef1 here. */
    b=(b&0xffff)+(b>>16)*(65536-MOD_ADLER);
    if (b>=MOD_ADLER) b-=MOD_ADLER;
    return (b<<16)|a;
}



dword Adler32_test(byte *data,uint size)
{
	dword s1=1,s2=0;
	while(size--) {
		s1=(s1+*data++)%MOD_ADLER;
		s2=(s2+s1)%MOD_ADLER;
	}
	return (s2<<16)+s1;
}


void main()
{
	int n,size; dword res1,res2;
	byte data[]="3,1415926535897932384626433832795";
	if(sizeof(dword)!=4 || sizeof(byte)!=1) printf("type error\n\a"),getchar();
	res1=Adler32(data,strlen((char*)data));
	res2=Adler32_test(data,strlen((char*)data));
	if(res1!=0x726106c8 || res2!=0x726106c8) printf("Internal error!\n\a"),getchar();
	printf("done...\n");
}

